import objectdraw.*;
import java.awt.*;

/*
 * Example BigSnowman: a recursive object to create a many-leveled
 * snowman-like object.  Its base snowball is determined by
 * the point and diameter passed to the constructor, and successive
 * smaller snowballs are placed atop that one until they become too small
 *
 * Jim Teresco, The College of Saint Rose, CSC 252, Spring 2014
 *
 * $Id: BigSnowman.java 2333 2014-03-12 17:48:14Z terescoj $
 */

public class BigSnowman {

    // the minimum size of a snowball to be added as the "head"
    private static final double MIN_HEAD_SIZE = 25;
    
    // scaling factor for each smaller snowball
    private static final double SCALE_FACTOR = 0.7;
    
    // the slightly blue colors for the interior and borders of snowballs
    private static final Color SNOW_COLOR = new Color(240, 250, 255);
    private static final Color BORDER_COLOR = new Color(200, 200, 255);
    
    // our instance variables to represent the snowman
    // the 2 components that make up the base snowball
    private FilledOval base;
    private FramedOval baseFrame;
    
    // and an instance variable to keep track of any additional snowballs
    private BigSnowman rest;
    
    // construct one
    public BigSnowman(Location upperLeft, double diameter, DrawingCanvas canvas) {
        
        // The base is easy: Just construct our two graphical components
        base = new FilledOval(upperLeft, diameter, diameter, canvas);
        base.setColor(SNOW_COLOR);
        baseFrame = new FramedOval(upperLeft, diameter, diameter, canvas);
        baseFrame.setColor(BORDER_COLOR);
        
        // how big would the next snowball up be?
        double newDiameter = diameter * SCALE_FACTOR;
        // now, we construct the rest if it would be large enough to be displayed
        if (newDiameter >= MIN_HEAD_SIZE) {
            // how much to move right to center next snowball on this one?
            double xOffset = (diameter - newDiameter)/2;
            upperLeft.translate(xOffset, -newDiameter);
            rest = new BigSnowman(upperLeft, newDiameter, canvas);
        }
    }
    
    // we'll need contains and move to support dragging
    public boolean contains(Location point) {
        
        if (base.contains(point)) return true;
        
        if (rest != null) return rest.contains(point);
        
        return false;
   
    }
    
    public void move(double dx, double dy) {
        
        // move the pieces we're responsible for
        base.move(dx, dy);
        baseFrame.move(dx, dy);
        
        // and if there's more, move those recursively
        if (rest != null) {
            rest.move(dx, dy);
        }
    }
}
