import objectdraw.*;
import java.awt.*;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/*
 * Example BigSnowman: the window controller to make many-leveled
 * snowman-like objects on the canvas.
 * 
 * The game: 
 * - Press the mouse to create a new snowman whose base size
 * is determined by the JSlider
 * - Press on most recent snowman to drag him instead of creating
 * a new snowman
 *
 * Jim Teresco, The College of Saint Rose, CSC 252, Spring 2014
 *
 * $Id: BigSnowmanController.java 2333 2014-03-12 17:48:14Z terescoj $
 */

public class BigSnowmanController extends WindowController 
    implements ChangeListener {

    // keep track of the newest snowman
    private BigSnowman newestSnowman;
    
    // valid range of snowman base sizes
    private static final int MIN_SIZE = 35;
    private static final int MAX_SIZE = 200;
    
    // our Swing components: a slider and label for current selected size
    private JLabel sizeDisplay;
    private JSlider sizer;
    
    // standard dragging code
    private boolean dragging;
    private Location lastMouse;
    
    // set up GUI
    public void begin() {
        
        // Swing setup
        Container contentPane = getContentPane();
        
        // we'll put our slider and label up north
        JPanel northPanel = new JPanel();
        
        // label and slider into the panel
        sizeDisplay = new JLabel("" + MIN_SIZE);
        northPanel.add(sizeDisplay);
        
        sizer = new JSlider(JSlider.HORIZONTAL, MIN_SIZE, MAX_SIZE, MIN_SIZE);
        northPanel.add(sizer);
        
        // put our panel in the north
        contentPane.add(northPanel, BorderLayout.NORTH);
        
        // and we'll listen for slider changes so we can update the label
        sizer.addChangeListener(this);
        
        contentPane.validate();
    }
    
    public void onMousePress(Location point) {
    
        // drag if we've pressed on the newest snowman
        if (newestSnowman != null && newestSnowman.contains(point)) {
            dragging = true;
            lastMouse = point;
        }
        // create a new one otherwise
        else {
            newestSnowman = new BigSnowman(point, sizer.getValue(), canvas);
            dragging = false;
        }
    }

    public void onMouseDrag(Location point) {
    
        if (dragging) {
            newestSnowman.move(point.getX() - lastMouse.getX(),
                               point.getY() - lastMouse.getY());
            lastMouse = point;
        }
    }
    
    public void onMouseRelease(Location point) {
    
        if (dragging) {
            newestSnowman.move(point.getX() - lastMouse.getX(),
                               point.getY() - lastMouse.getY());
            lastMouse = point;
        }
    }
       
    // our change listener
    public void stateChanged(ChangeEvent e) {
    
        sizeDisplay.setText("" + sizer.getValue());
    }
    
}
